#!/usr/bin/python3

import unittest
import os
import sys

# Add parent to the start of os.path, import ucammap
sys.path.insert(0,os.path.join(os.path.dirname(__file__),"..")) 
import ucammap

address_data = [
    {'type': 'location', 'location': 987},
    {'type': 'site',     'site': 'New Museums Site'}, 
    {'type': 'number',   'number': '123'},
    {'type': 'number',   'number': '456'},
    {'type': 'street',   'street': 'Pembroke Street'}, 
    {'type': 'place',    'place': 'Cambridge'}, 
    {'type': 'postcode', 'postcode': 'CB2 3QH'},
    {'type': 'number',   'number': '999'},
    {'type': 'address',  'address': 'Some unparsed text'}
    ]

class AddressTestCase(unittest.TestCase):

    def testPopulated(self):
        """General tests on a populated address object"""

        addr = ucammap.Address(address_data)

        self.assertEqual (addr.raw_data, address_data)

        self.assertIs    (type(addr.as_block()),list)
        self.assertEqual (len(addr.as_block()),5)

        self.assertEqual (addr.as_line(),
           'New Museums Site, 123 456 Pembroke Street, Cambridge, CB2 3QH, 999')

        self.assertEqual (str(addr),
           'New Museums Site, 123 456 Pembroke Street, Cambridge, CB2 3QH, 999')

    def testEmpty(self):
        """General tests on an empty address"""

        empty = ucammap.Address([])

        self.assertEqual (empty.raw_data, [])

        self.assertIs    (type(empty.as_block()),list)
        self.assertEqual (len(empty.as_block()),0)

        self.assertEqual (empty.as_line(),'')

    def testUnstructured(self):
        """Tests on an address only containing unstructured data"""

        partial_data = [
            {'type': 'location', 'location': 987},
            {'type': 'address',  'address': 'Some unparsed text'}
            ]
        partial = ucammap.Address(partial_data)

        self.assertEqual (partial.raw_data, partial_data)
        self.assertIs    (type(partial.as_block()),list)
        self.assertEqual (len(partial.as_block()),0)
        self.assertEqual (partial.as_line(),'Some unparsed text')

    def testExceptions(self):
        """Test address exceptions"""

        # A address containing an entry with no type throws an error
        with self.assertRaisesRegexp(ucammap.APIError,
                                     'Type field missing') as cm:
            c = ucammap.Address([{'typeZZZ': 'location', 'location': 987}])

        # A address containing an entry with no data field throws an error
        with self.assertRaisesRegexp(ucammap.APIError,
                                     'Data field "location" missing') as cm:
            c = ucammap.Address([{'type': 'location', 'locationZZZ': 987}])


if __name__ == '__main__':
    unittest.main()
