#!/usr/bin/python3

import unittest
import os
import sys

import collections

# Add parent to the start of os.path, import ucammap
sys.path.insert(0,os.path.join(os.path.dirname(__file__),"..")) 
import ucammap

info_data = {
    'type': 'url',
    'url': 'http://www.ucs.cam.ac.uk/about-us/contact/general', 
    'role': 'contacts', 
    'options': 'foo'}

info_minimal = {
    'type': 'url',
    'url': 'http://www.ucs.cam.ac.uk/about-us/contact/general'} 

attributes = {'type', 'url', 'role', 'options','value'}

class AddressTestCase(unittest.TestCase):

    def setUp(self):
        self.info    = ucammap.Info(info_data)
        self.minimal = ucammap.Info(info_minimal)

    def testType(self):
        """Check object types"""

        for obj in (self.info,self.minimal):
            self.assertTrue(isinstance(obj,ucammap.Info))

    def testRawData(self):
        """Tet raw_data retrieval"""

        self.assertEqual (self.info.raw_data, info_data)
        self.assertEqual (self.minimal.raw_data, info_minimal)

    def testAttributes(self):
        """Test attribute retrieval"""

        self.assertEqual (self.info.type,'url')

        self.assertEqual (self.info.value,
                           'http://www.ucs.cam.ac.uk/about-us/contact/general') 

        self.assertEqual (self.info.role,'contacts')

        self.assertEqual (self.info.options,'foo')

        self.assertEqual (str(self.info),
           'Url: http://www.ucs.cam.ac.uk/about-us/contact/general (contacts)')

        # Non-existent attributes raises correct error
        with self.assertRaisesRegexp(AttributeError,'nonsuch'):
            c = self.info.nonsuch

        # Missing attributes from empty return none...
        for attribute in ('role', 'options'):
            self.assertIsNone(getattr(self.minimal,attribute))

        # ...but non-existent attributes raise AttributeError
        with self.assertRaisesRegexp(AttributeError,'nonsuch'):
            c = self.info.nonsuch

    def testAddress(self):
        """Test an info object containing an address"""

        info_address_data = {
            'type': 'address', 
            'address':  [
                {'type': 'address',  'address': 'Some unparsed text'} ] }
        info_address = ucammap.Info(info_address_data)

        self.assertEqual (info_address.raw_data, info_address_data)
        self.assertEqual (info_address.type,'address')
        self.assertEqual (type(info_address.value), ucammap.Address)

    def testExceptions(self):
        """Test Info exceptions"""

        # A info with no type throws an error
        with self.assertRaisesRegexp(ucammap.APIError,
                                             'Type field missing'):
            c = ucammap.Info({'typeZZZ': 'url',
                  'url': 'http://www.ucs.cam.ac.uk/about-us/contact/general', 
                  'role': 'contacts', 'options': 'foo'})

        # A info record without the right data key  throws an error
        with self.assertRaisesRegexp(ucammap.APIError,
                                             'Data field "url" missing'):
            c = ucammap.Info({'type': 'url',
                 'urlZZZ': 'http://www.ucs.cam.ac.uk/about-us/contact/general', 
                 'role': 'contacts', 'options': 'foo'})

if __name__ == '__main__':
    unittest.main()
